==> gps-0.93.tar.gz: gps-0.93/src/triplet.h <==
/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __TRIPLET_H
#define __TRIPLET_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <iostream>
#include <string>
#include <sstream>
#include <stdexcept>
#include "read.h"
#include "defs.h"
//#include <sqlplus.hh>
//#include <custom.hh>

//sql_create_basic_3(Triplet,0,0,std::string,client_address,std::string,sender,std::string,recipient)

/**
@brief Triplet representation of (sender,recipient,client_address) triplet
@author Michael Moritz
*/
#ifndef OLDWEAK
class Triplet
{
public:
	typedef enum {
		TM_sender,
		TM_recipient,
		TM_client
	}TripletMember;
	Triplet(const Reader& reader)
	: _client_address(reader.getValue(ATTR_CLIENT)), 
	  _sender(reader.getValue(ATTR_SENDER)), 
	  _recipient(reader.getValue(ATTR_RECIPIENT)),
	  _client_address_numeric()
	{ 
	  	convertClientAddress();
	}
	Triplet(const std::string& client,const std::string& sender,const std::string& reciepient,
		const std::string& client_numeric="") 
	: 	_client_address(client), 
		_sender(sender), 
		_recipient(reciepient),
		_client_address_numeric(client_numeric) 
	{
		if(!_client_address_numeric.size())
		  	convertClientAddress();
	}
	const std::string& getClientAddress() const { return _client_address; }
	const std::string& getClientAddressNumeric() const { return _client_address_numeric ; }
	
	///utility function for messsages etc.
	const std::string getClientAddressNumericStr(const char sep='.') const { 
		std::stringstream stm;
		unsigned size = getClientAddressNumeric().size();
		//TODO check if this way of conversion is portable
		const unsigned char* pu = reinterpret_cast<const unsigned char*>(_client_address_numeric.c_str());
		for(unsigned i=0;i < size-1;i++) 
			stm << static_cast<unsigned>(pu[i]) << sep;
		if(size >= 1)
			stm << static_cast<unsigned>(pu[size-1]);
		return stm.str();
	}
	///convenience function for SQL update, replace
	const std::string getClientAddressNumericStrSQL(const std::string& sep,const std::string& base,
		unsigned start,const std::string& afterstart,unsigned mult,unsigned size) const { 
		std::stringstream stm;
		if(size == 0)
			size = getClientAddressNumeric().size();
		//TODO check if this way of conversion is portable
		const unsigned char* pu = reinterpret_cast<const unsigned char*>(_client_address_numeric.c_str());
		for(unsigned i=0;i < size-1;i++) {
			stm << base << start << afterstart << static_cast<unsigned>(pu[i]) << sep;
			start /= mult;
		}
		if(size >= 1)
			stm << base << start << afterstart << static_cast<unsigned>(pu[size-1]);
		return stm.str();
	}
	const std::string& getSender() const { return _sender; }
	const std::string& getRecipient() const { return _recipient; }
	const std::string& getMember(TripletMember m) const throw (std::out_of_range){
		switch(m) {
			case TM_sender:
				return _sender;
			case TM_recipient:
				return _recipient;
			case TM_client:
				return _client_address;
			default:
				throw std::out_of_range( "Triplet::getMember out of range" );			
		}
	}
protected:
	void convertClientAddress() 
	{
		const std::string& tStr = _client_address;
		//std::cout << "tStr" << tStr << std::endl;
		std::string::size_type i = tStr.find('.');
		std::string::size_type j = 1; //skip '
		if(tStr[0] != '\'') //unquoted
			j = 0;
		if(i != std::string::npos) {
			do {
				//std::cout << "substr " << tStr.substr(j,i-j) << std::endl;
				_client_address_numeric += atoi((tStr.substr(j,i-j)).c_str());
				j = i+1;
				i = tStr.find('.',j);
				if(i == std::string::npos) {
					i = tStr.find('\'',j);
					if((i == std::string::npos) && (tStr[0] != '\'')) { //unquoted
						//std::cout << "substr " << tStr.substr(j,i) << std::endl;
						_client_address_numeric += atoi((tStr.substr(j,i)).c_str());
						break;
					}
				}
			}while( i != std::string::npos);
		}
	}
private:
	std::string _client_address;
	std::string _sender;
	std::string _recipient;
	std::string _client_address_numeric; //not pretty
};
class WLCompare
{
public:
	const char* _compareWith;
	const char* _compareOp;
	Triplet::TripletMember _tripletMember;
	const char* _compareBinder;
};
typedef WLCompare WLCompares[MAX_WL_COMPARES];

#else //OLDWEAK

class Triplet
{
public:
	typedef enum {
		TM_sender,
		TM_recipient,
		TM_client,
		TM_weakclient
	}TripletMember;
	Triplet(const Reader& reader)
	: _client_address(reader.getValue(ATTR_CLIENT)), 
	  _sender(reader.getValue(ATTR_SENDER)), 
	  _recipient(reader.getValue(ATTR_RECIPIENT)),
	  _weakclient(),
	  _client_address_numeric()
	  { }
	Triplet(const std::string& client,const std::string& sender,const std::string& reciepient,
		const std::string& weakclient="",const std::string& client_numeric="") 
	: 	_client_address(client), 
		_sender(sender), 
		_recipient(reciepient),
		_weakclient(weakclient),
		_client_address_numeric(client_numeric) 
		{}
	const std::string& getClientAddress() const { return _client_address; }
	const std::string& getClientAddressNumeric() const { 
		if(_client_address_numeric.size())
			return _client_address_numeric;
		else {
			if( !_client_address.size() )
				return _client_address_numeric;
			const std::string& tStr = _client_address;
//			std::cout << "tStr" << tStr << std::endl;
			std::string::size_type i = tStr.find('.');
			std::string::size_type j = 1; //skip '
			if(tStr[0] != '\'') //unquoted
				j = 0;
			if(i != std::string::npos) {
				do {
					//std::cout << "substr " << tStr.substr(j,i-j) << std::endl;
					_client_address_numeric += atoi((tStr.substr(j,i-j)).c_str());
					j = i+1;
					i = tStr.find('.',j);
					if(i == std::string::npos) {
						i = tStr.find('\'',j);
						if((i == std::string::npos) && (tStr[0] != '\'')) { //unquoted
							//std::cout << "substr " << tStr.substr(j,i) << std::endl;
							_client_address_numeric += atoi((tStr.substr(j,i)).c_str());
							break;
						}
					}
				}while( i != std::string::npos);
			}
			return _client_address_numeric;
		}
	};
	///utility function for messsages etc.
	const std::string getClientAddressNumericStr(const char sep='.') const { 
		std::stringstream stm;
		unsigned size = getClientAddressNumeric().size();
		//TODO check if this way of conversion is portable
		const unsigned char* pu = reinterpret_cast<const unsigned char*>(_client_address_numeric.c_str());
		for(unsigned i=0;i < size-1;i++) 
			stm << static_cast<unsigned>(pu[i]) << sep;
		if(size >= 1)
			stm << static_cast<unsigned>(pu[size-1]);
		return stm.str();
	}
	///convenience function for SQL update, replace
	const std::string getClientAddressNumericStrSQL(const char sep,const std::string& base,
		unsigned start,const std::string& afterstart,unsigned mult) const { 
		std::stringstream stm;
		unsigned size = getClientAddressNumeric().size();
		//TODO check if this way of conversion is portable
		const unsigned char* pu = reinterpret_cast<const unsigned char*>(_client_address_numeric.c_str());
		for(unsigned i=0;i < size-1;i++) {
			stm << base << start << afterstart << static_cast<unsigned>(pu[i]) << sep;
			start /= mult;
		}
		if(size >= 1)
			stm << base << start << afterstart << static_cast<unsigned>(pu[size-1]);
		return stm.str();
	}
	const std::string& getSender() const { return _sender; }
	const std::string& getRecipient() const { return _recipient; }
	const std::string& getMember(TripletMember m) const throw (std::out_of_range){
		switch(m) {
			case TM_sender:
				return _sender;
			case TM_recipient:
				return _recipient;
			case TM_client:
				return _client_address;
			case TM_weakclient:
				return _weakclient;
			default:
				throw std::out_of_range( "Triplet::getMember out of range" );			
		}
	}
private:
	std::string _client_address;
	std::string _sender;
	std::string _recipient;
	std::string _weakclient;
	mutable std::string _client_address_numeric; //not pretty
};
class WLCompare
{
public:
	const char* _compareWith;
	const char* _compareOp;
	Triplet::TripletMember _tripletMember;
	const char* _compareBinder;
};
typedef WLCompare WLCompares[MAX_WL_COMPARES];
#endif //OLDWEAK

#endif

==> gps-1.004.tar.gz: gps-1.004/src/triplet.h <==
/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __TRIPLET_H
#define __TRIPLET_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <iostream>
#include <string>
#include <sstream>
#include <stdexcept>
#include <netdb.h>
#include "read.h"
#include "defs.h"

#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

//sql_create_basic_3(Triplet,0,0,std::string,client_address,std::string,sender,std::string,recipient)

/**
@brief Triplet representation of (sender,recipient,client_address) triplet
@author Michael Moritz
*/

class Triplet
{
public:
	typedef enum {
		TM_sender,
		TM_recipient,
		TM_client,
		TM_resolved,
		TM_quotedweakclient
	}TripletMember;
	Triplet(const Reader& reader)
	: _client_address(reader.getValue(ATTR_CLIENT)), 
	  _sender(reader.getValue(ATTR_SENDER)), 
	  _recipient(reader.getValue(ATTR_RECIPIENT)),
	  _resolved(),
	  _client_address_numeric(),
	  _not_resolved(),
	  _weakclient()
	  { }
	Triplet(const std::string& client,const std::string& sender,const std::string& reciepient,
		const std::string& client_numeric="") 
	: 	_client_address(client), 
		_sender(sender), 
		_recipient(reciepient),
		_resolved(),
		_client_address_numeric(client_numeric),
		_not_resolved(),
		_weakclient()
		{}
	const std::string& getClientAddress() const { return _client_address; }
	const std::string& getClientAddressNumeric() const { 
		if(_client_address_numeric.size())
			return _client_address_numeric;
		else {
			if( !_client_address.size() )
				return _client_address_numeric;
			const std::string& tStr = _client_address;
//			std::cout << "tStr" << tStr << std::endl;
			std::string::size_type i = tStr.find('.');
			std::string::size_type j = 1; //skip '
			if(tStr[0] != '\'') //unquoted
				j = 0;
			if(i != std::string::npos) {
				do {
					//std::cout << "substr " << tStr.substr(j,i-j) << std::endl;
					_client_address_numeric += atoi((tStr.substr(j,i-j)).c_str());
					j = i+1;
					i = tStr.find('.',j);
					if(i == std::string::npos) {
						i = tStr.find('\'',j);
						if((i == std::string::npos) && (tStr[0] != '\'')) { //unquoted
							//std::cout << "substr " << tStr.substr(j,i) << std::endl;
							_client_address_numeric += atoi((tStr.substr(j,i)).c_str());
							break;
						}
					}
				}while( i != std::string::npos);
			}
			return _client_address_numeric;
		}
	};
	///utility function for messsages etc.
	const std::string getClientAddressNumericStr(const char sep='.',unsigned size=0) const { 
		std::stringstream stm;
		if(size == 0)
			size = getClientAddressNumeric().size();
		//TODO check if this way of conversion is portable
		const unsigned char* pu = reinterpret_cast<const unsigned char*>(_client_address_numeric.c_str());
		for(unsigned i=0;i < size-1;i++) 
			stm << static_cast<unsigned>(pu[i]) << sep;
		if(size >= 1)
			stm << static_cast<unsigned>(pu[size-1]);
		return stm.str();
	}
	///convenience function for SQL update, replace
	const std::string getClientAddressNumericStrSQL(const std::string& sep,const std::string& base,
		unsigned start,const std::string& afterstart,unsigned mult,unsigned size) const { 
		std::stringstream stm;
//		unsigned size = bytes;
		if(size == 0)
			size = getClientAddressNumeric().size();
		//TODO check if this way of conversion is portable
		const unsigned char* pu = reinterpret_cast<const unsigned char*>(getClientAddressNumeric().c_str());
		for(unsigned i=0;i < size-1;i++) {
			stm << base << start << afterstart << static_cast<unsigned>(pu[i]) << sep;
			start /= mult;
		}
		if(size >= 1)
			stm << base << start << afterstart << static_cast<unsigned>(pu[size-1]);
		return stm.str();
	}
	//return resolved host name or empty string if error
	const std::string getResolved() const {
		if(_resolved.size() > 0)
			return _resolved;
		if(_not_resolved.size() > 0)
			return _resolved; //empty string
		const std::string& clientAddressNumeric = this->getClientAddressNumeric();
		//would like to use getnameinfo but documentation sucks big time
		hostent* hp = ::gethostbyaddr(clientAddressNumeric.c_str(),
			      clientAddressNumeric.size(),
			      AF_INET);
		if(hp) {
			_resolved = hp->h_name;
		} else {
			std::string err;
			switch(h_errno) {
				case HOST_NOT_FOUND:
					err = "HOST_NOT_FOUND";
					break;
				case NO_ADDRESS:
					err = "NO_ADDRESS|NO_DATA";
					break;
				case NO_RECOVERY:
					err = "NO_RECOVERY";
					break;
				case TRY_AGAIN:
					err = "TRY_AGAIN";
					break;
				default:
					err = "Unexpected";
					break;
			}
			_not_resolved = err;
		}
		return _resolved;
	}
	const std::string& getResolveError() const { return _not_resolved; }
	const std::string& getSender() const { return _sender; }
	const std::string& getRecipient() const { return _recipient; }
	const std::string& getMember(TripletMember m) const throw (std::out_of_range){
		switch(m) {
			case TM_sender:
				return _sender;
			case TM_recipient:
				return _recipient;
			case TM_client:
				return _client_address;
			case TM_resolved:
				return _resolved;
			case TM_quotedweakclient:
				return getQuotedWeakClient();
			default:
				throw std::out_of_range( "Triplet::getMember out of range" );			
		}
	}
	const std::string& getQuotedWeakClient() const {
		if(_weakclient.size() == 0)
			_weakclient = "'" + getClientAddressNumericStr('.',3)+"." + "'";
		return _weakclient;
	}
private:
	/// client IP address e.g. 10.0.0.1
	std::string _client_address;
	/// sender's e-mail address someone@something.somedomain
	std::string _sender;
	/// reciepient's e-mail address someone@something.somedomain
	std::string _recipient;
	/// the following members get calculated from the triplet
	/// client addressed after reverse DNS lookup
	mutable std::string _resolved;
	/// client IP address byte-by-byte e.g. [ 10,0,0,1 ] 
	mutable std::string _client_address_numeric; //not pretty
	/// empty if resolved or not resolved yet, otherwise hold reason
	mutable std::string _not_resolved;
	/// quoted weak client ip address, trimmed last byte e.g. '10.0.0.'
	mutable std::string _weakclient;
};
class WLCompare
{
public:
	const char* _compareWith;
	const char* _compareOp;
	Triplet::TripletMember _tripletMember;
	const char* _compareBinder;
};
typedef WLCompare WLCompares[MAX_WL_COMPARES];

#endif
